@@ -216,7 +216,10 @@ static char **get_groupnames(void)
 
 static bool name_is_in_groupnames(char *name, char **groupnames)
 {
-	while (groupnames) {
+	if (!groupnames)
+		return false;
+
+	while (*groupnames) {
 		if (!strcmp(name, *groupnames))
 			return true;
 		groupnames++;
