@@ -113,14 +113,14 @@ static const char AA_PROFILE_BASE[] =
 "  deny /sys/kernel/debug/{,**} rwklx,\n"
 "\n"
 "  # allow paths to be made dependent, shared, private or unbindable\n"
-"  mount options=(rw,make-slave) -> **,\n"
-"  mount options=(rw,make-rslave) -> **,\n"
-"  mount options=(rw,make-shared) -> **,\n"
-"  mount options=(rw,make-rshared) -> **,\n"
-"  mount options=(rw,make-private) -> **,\n"
-"  mount options=(rw,make-rprivate) -> **,\n"
-"  mount options=(rw,make-unbindable) -> **,\n"
-"  mount options=(rw,make-runbindable) -> **,\n"
+"  mount options=(rw,make-slave) -> /{,**},\n"
+"  mount options=(rw,make-rslave) -> /{,**},\n"
+"  mount options=(rw,make-shared) -> /{,**},\n"
+"  mount options=(rw,make-rshared) -> /{,**},\n"
+"  mount options=(rw,make-private) -> /{,**},\n"
+"  mount options=(rw,make-rprivate) -> /{,**},\n"
+"  mount options=(rw,make-unbindable) -> /{,**},\n"
+"  mount options=(rw,make-runbindable) -> /{,**},\n"
 "\n"
 "  # allow bind-mounts of anything except /proc, /sys and /dev\n"
 "  mount options=(rw,bind) /[^spd]*{,/**},\n"
@@ -170,6 +170,9 @@ static const char AA_PROFILE_BASE[] =
 "  mount options=(rw,move) /s[^y]*{,/**},\n"
 "  mount options=(rw,move) /sy[^s]*{,/**},\n"
 "  mount options=(rw,move) /sys?*{,/**},\n"
+"\n";
+
+static const char AA_PROFILE_BASE_NO_NESTING[] =
 "\n"
 "  # generated by: lxc-generate-aa-rules.py container-rules.base\n"
 "  deny /proc/sys/[^kn]*{,/**} wklx,\n"
@@ -336,14 +339,14 @@ static const char AA_PROFILE_UNPRIVILEGED[] =
 "  pivot_root,\n"
 "\n"
 "  # Allow modifying mount propagation\n"
-"  mount options=(rw,make-slave) -> **,\n"
-"  mount options=(rw,make-rslave) -> **,\n"
-"  mount options=(rw,make-shared) -> **,\n"
-"  mount options=(rw,make-rshared) -> **,\n"
-"  mount options=(rw,make-private) -> **,\n"
-"  mount options=(rw,make-rprivate) -> **,\n"
-"  mount options=(rw,make-unbindable) -> **,\n"
-"  mount options=(rw,make-runbindable) -> **,\n"
+"  mount options=(rw,make-slave) -> /{,**},\n"
+"  mount options=(rw,make-rslave) -> /{,**},\n"
+"  mount options=(rw,make-shared) -> /{,**},\n"
+"  mount options=(rw,make-rshared) -> /{,**},\n"
+"  mount options=(rw,make-private) -> /{,**},\n"
+"  mount options=(rw,make-rprivate) -> /{,**},\n"
+"  mount options=(rw,make-unbindable) -> /{,**},\n"
+"  mount options=(rw,make-runbindable) -> /{,**},\n"
 "\n"
 "  # Allow all bind-mounts\n"
 "  mount options=(rw,bind),\n"
@@ -755,6 +758,10 @@ static char *get_apparmor_profile_content(struct lsm_ops *ops, struct lxc_conf *
 	must_append_sized(&profile, &size, AA_PROFILE_BASE,
 	                  STRARRAYLEN(AA_PROFILE_BASE));
 
+	if (!conf->lsm_aa_allow_nesting)
+		must_append_sized(&profile, &size, AA_PROFILE_BASE_NO_NESTING,
+		                  STRARRAYLEN(AA_PROFILE_BASE_NO_NESTING));
+
 	append_all_remount_rules(&profile, &size);
 
 	if (ops->aa_supports_unix)
@@ -768,8 +775,10 @@ static char *get_apparmor_profile_content(struct lsm_ops *ops, struct lxc_conf *
 	if (ops->aa_can_stack && !ops->aa_is_stacked) {
 		char *namespace, *temp;
 
-		must_append_sized(&profile, &size, AA_PROFILE_STACKING_BASE,
-		                  STRARRAYLEN(AA_PROFILE_STACKING_BASE));
+
+		if (!conf->lsm_aa_allow_nesting)
+			must_append_sized(&profile, &size, AA_PROFILE_STACKING_BASE,
+			                  STRARRAYLEN(AA_PROFILE_STACKING_BASE));
 
 		namespace = apparmor_namespace(conf->name, lxcpath);
 		temp = must_concat(NULL, "  change_profile -> \":", namespace, ":*\",\n"
@@ -779,7 +788,7 @@ static char *get_apparmor_profile_content(struct lsm_ops *ops, struct lxc_conf *
 
 		must_append_sized(&profile, &size, temp, strlen(temp));
 		free(temp);
-	} else {
+	} else if (!conf->lsm_aa_allow_nesting) {
 		must_append_sized(&profile, &size, AA_PROFILE_NO_STACKING,
 		                  STRARRAYLEN(AA_PROFILE_NO_STACKING));
 	}
